/*
 * Autorzy: 
 *      Marek Matejko, Krzysztof Mazur, Dawid Paszkot
 * Opis:
 *		Porownanie szybkosci i latwosci w implementacji wybranych algorytmow sortujacych
 */

#include <iostream>
#include <chrono>
#include <vector>
#include <algorithm>

using namespace std;

// sortowanie bąbelkowe
void sortowanie_babelkowe(int tablica[], int rozmiar) 
{
    for (int i=1; i<rozmiar; i++)
        for (int j=rozmiar-1; j>=i; j--)
            if (tablica[j]<tablica[i])
            {
                int tmp=tablica[j-1];
                tablica[j-1]=tablica[j];
                tablica[j] = tmp;
            }
}
// ***********************************

// sortowanie kubelkowe
void sortowanie_kubelkowe(float tablica[], int rozmiar)
{
    int k = 99999/rozmiar;
    vector<float> kub[k];

    for(int i=0; i<rozmiar; i++)
    {
        int nr = k*tablica[i];
        kub[nr].push_back(tablica[i]);
    }

    for (int i=0; i<k; i++)
    {
        sort(kub[i].begin(), kub[i].end());   
    } 
}
// ***********************************

// sortowanie szybkie
void sortowanie_szybkie(int *tab, int lewy, int prawy)
{
	if(prawy <= lewy) return;
	
	int i = lewy - 1, j = prawy + 1, 
	srodek = tab[(lewy+prawy)/2];
	
	while(true)
	{
		while(srodek>tab[++i]);
		while(srodek<tab[--j]);
		if( i <= j)
			swap(tab[i],tab[j]);
		else
			break;
	}
	if(j > lewy)
	    sortowanie_szybkie(tab, lewy, j);
	if(i < prawy)
	    sortowanie_szybkie(tab, i, prawy);
}
// ***********************************

// funkcja glowna
int main() 
{
    // generowanie losowych liczb
    int n = 1000; // liczba elementów tablicy
    int tablica_losowych_liczb[n];
    int tablica_A[n];
    int tablica_B[n];
    float tablica_C[n];

    for (int i=0; i<n; i++) 
    {
        tablica_losowych_liczb[i] = rand() % 99999 + 10000;
        tablica_A[i] = tablica_losowych_liczb[i];
        tablica_B[i] = tablica_losowych_liczb[i];
        tablica_C[i] = (float)tablica_losowych_liczb[i];
    }
    // ***********************************

    // mierzenie czasu sorotwania bobelkowego
    auto bubble_czas_roz= chrono::high_resolution_clock::now();
    sortowanie_babelkowe (tablica_A, n); // sortowanie bąbelkowe
    auto bubble_czas_zak = chrono::high_resolution_clock::now();

    auto bubble_czas_wyk = bubble_czas_zak - bubble_czas_roz; // sprawdzic zmienne
    cout << "### SORTOWANIE BABELKOWE ###\n";
    cout << "Ilosc elementow: " << n << "\n";
    cout << "Czas wykonania: " << bubble_czas_wyk/chrono::milliseconds(1)<< " ms\n" << endl;
    // ***********************************

    // mierzenie czasu sorotwania kubelkowe
    auto bucket_czas_roz = chrono::high_resolution_clock::now();
    sortowanie_kubelkowe (tablica_C, n); // sortowanie szybkie
    auto bucket_czas_zak = chrono::high_resolution_clock::now();

    auto bucket_czas_wyk = bucket_czas_zak - bucket_czas_roz; // sprawdzic zmienne
    cout << "### SORTOWANIE KUBEŁKOWE ###\n";
    cout << "Ilosc elementow: " << n << "\n";
    cout << "Czas wykonania: " << bucket_czas_wyk/chrono::milliseconds(1)<< " ms\n" << endl;
    // ***********************************

    // mierzenie czasu sorotwania szybkiego
    auto quick_czas_roz = chrono::high_resolution_clock::now();
    sortowanie_szybkie (tablica_B, 0, n); // sortowanie szybkie
    auto quick_czas_zak = chrono::high_resolution_clock::now();

    auto quick_czas_wyk = quick_czas_zak - quick_czas_roz; // sprawdzic zmienne
    cout << "### SORTOWANIE SZYBKIE ###\n";
    cout << "Ilosc elementow: " << n << "\n";
    cout << "Czas wykonania: " << quick_czas_wyk/chrono::milliseconds(1)<< " ms\n" << endl;
    // ***********************************
}
// ***********************************